'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var React = require('react');
var React__default = _interopDefault(React);
var P5Wrapper = _interopDefault(require('react-p5-wrapper'));

/* eslint-disable no-multi-assign */

/* eslint-disable no-plusplus */

/* eslint-disable no-underscore-dangle */

/* eslint-disable no-param-reassign */
function sketch(p) {
  var particles = [];
  var field = [];
  var fieldStep;
  var gravity;
  var config;
  var colorSet = [];

  p.myCustomRedrawAccordingToNewPropsHandler = function (props) {
    if (props.config) config = props.config;
    if (props.colorSet) colorSet = props.colorSet;
    p.setup();
  };

  var Particle = /*#__PURE__*/function () {
    function Particle(x, y, size, index) {
      this.baseSize = size;
      this.index = index || 0;
      this.spawn = p.createVector(x, y);
      this.init();
    }

    var _proto = Particle.prototype;

    _proto.init = function init() {
      var _config;

      this.size = this.baseSize * p.random(0.5, 1.5);
      this.start = p.millis();
      this.position = this.spawn.copy();
      this.velocity = p.createVector(0, 0);
      this.acceleration = p.createVector(0, 0);
      this.duration = ((_config = config) == null ? void 0 : _config.lifeSpan) * p.random(0.2, 1.2);
      this.drag = p.random(0.9, 1);
      this.addForce(p.constructor.Vector.fromAngle(p.random(p.TWO_PI), p.random(10)));
      this.color = p.random(colorSet);
    };

    _proto.display = function display() {
      var _config2;

      var s = 1;

      if (p.millis() - this.start < this.duration * 0.1) {
        s = p.map(p.millis() - this.start, 0, this.duration * 0.1, 0, 1);
      } else if (p.millis() - this.start > this.duration * 0.5) {
        s = p.map(p.millis() - this.start, this.duration * 0.5, this.duration, 1, 0);
      }

      p.fill(this.color);
      p.circle(this.position.x, this.position.y, this.size * s * p.map(this.velocity.mag(), 0, (_config2 = config) == null ? void 0 : _config2.topSpeed, 0.5, 1.2));
    };

    _proto.update = function update() {
      var _config3;

      this.velocity.add(this.acceleration);
      this.velocity.limit((_config3 = config) == null ? void 0 : _config3.topSpeed);
      this.velocity.mult(this.drag);
      this.position.add(this.velocity.copy().mult(1 / p._targetFrameRate));
      this.acceleration.mult(0);

      if (this.position.y > p.height || p.millis() - this.start > this.duration) {
        this.init();
      }
    };

    _proto.addForce = function addForce(vector) {
      this.acceleration.add(vector);
    };

    return Particle;
  }();

  function setGravity() {
    var _config4, _config4$gravity, _config5;

    gravity = p.constructor.Vector.fromAngle(p.radians((_config4 = config) == null ? void 0 : (_config4$gravity = _config4.gravity) == null ? void 0 : _config4$gravity.direction), (_config5 = config) == null ? void 0 : _config5.gravity.force);
  }

  function init() {
    var _config6, _config7;

    p.clear();
    p.fill(0);
    p.textSize((_config6 = config) == null ? void 0 : _config6.textSize);
    p.textAlign(p.CENTER, p.CENTER);
    p.textStyle(p.BOLD);
    p.text((_config7 = config) == null ? void 0 : _config7.text, p.width / 2, p.height / 2);
    p.noFill();
    particles = [];
    var step = p.floor(p.max(p.width, p.height) / p.min(160, p.min(p.width, p.height)));
    var i = 0;

    for (var x = 0; x < p.width; x += step) {
      for (var y = 0; y < p.height; y += step) {
        var targetX = x + step / 2;
        var targetY = y + step / 2;
        var alpha = p.get(targetX, targetY)[3];

        if (alpha > 0.5) {
          particles.push(new Particle(targetX, targetY, step * 3, i));
          i++;
        }
      }
    }

    field = {};
    p.clear();
    step = fieldStep = p.floor(p.max(p.width, p.height) / p.min(20, p.min(p.width, p.height)));
    i = 0;

    for (var _x = 0; _x < p.width; _x += step) {
      for (var _y = 0; _y < p.height; _y += step) {
        i++;
        var a = p.noise(i) * p.TWO_PI;
        field[_x + "-" + _y] = a;
        p.translate(_x, _y);
        p.rotate(a);
        p.rect(-step / 4, -step / 2, step / 2, step);
        p.resetMatrix();
      }
    }

    p.clear();

    p.draw = function () {
      var _config8, _config8$canvas;

      p.background(((_config8 = config) == null ? void 0 : (_config8$canvas = _config8.canvas) == null ? void 0 : _config8$canvas.bg) || 'white');
      particles.forEach(function (particle) {
        var _config9, _config10;

        particle.addForce(gravity);
        particle.addForce(p.constructor.Vector.fromAngle(field[particle.position.x - particle.position.x % fieldStep + "-" + (particle.position.y - particle.position.y % fieldStep)] + ((_config9 = config) == null ? void 0 : _config9.flowOffset), (_config10 = config) == null ? void 0 : _config10.flow));
        particle.update();
        particle.display();
      });
    };
  }

  p.setup = function () {
    var _config11, _config11$canvas, _config12, _config12$canvas;

    p.createCanvas((_config11 = config) == null ? void 0 : (_config11$canvas = _config11.canvas) == null ? void 0 : _config11$canvas.width, (_config12 = config) == null ? void 0 : (_config12$canvas = _config12.canvas) == null ? void 0 : _config12$canvas.height);
    setGravity();
    init();
    p.frameRate(60);
    p.noStroke();
    p.colorMode(p.HSL, 100);
  };
}

var TextParticles = function TextParticles(_ref) {
  var text = _ref.text,
      textSize = _ref.textSize,
      flow = _ref.flow,
      topSpeed = _ref.topSpeed,
      lifeSpan = _ref.lifeSpan,
      flowOffset = _ref.flowOffset,
      gravity = _ref.gravity,
      canvas = _ref.canvas,
      colorSet = _ref.colorSet;

  var _useState = React.useState({
    colorSet: colorSet,
    config: {
      text: text,
      flow: flow,
      topSpeed: topSpeed,
      lifeSpan: lifeSpan,
      flowOffset: flowOffset,
      gravity: gravity,
      canvas: canvas,
      textSize: textSize
    },
    sketch: sketch
  }),
      state = _useState[0];

  return React__default.createElement(P5Wrapper, {
    sketch: state.sketch,
    config: state.config,
    colorSet: state.colorSet
  });
};

TextParticles.defaultProps = {
  text: 'H.A',
  textSize: 160,
  flow: 0.3,
  topSpeed: 100,
  lifeSpan: 2000,
  flowOffset: 0,
  gravity: {
    direction: 90,
    force: 0
  },
  canvas: {
    width: 300,
    height: 300,
    bg: 'white'
  },
  colorSet: ['#f44336', '#e91e63', '#9c27b0', '#673ab7', '#3f51b5', '#2196f3', '#03a9f4', '#00bcd4', '#009688', '#4CAF50', '#8BC34A', '#CDDC39', '#FFEB3B', '#FFC107', '#FF9800', '#FF5722']
};

exports.default = TextParticles;
//# sourceMappingURL=react-text-particles.cjs.development.js.map
